@@ -24,6 +24,25 @@ from .util import DataType, IOType, LibyangError, c2str, str2c
 LOG = logging.getLogger(__name__)
 
 
+# -------------------------------------------------------------------------------------
+def implicit_flags(
+    no_config: bool = False,
+    no_defaults: bool = False,
+    no_state: bool = False,
+    output: bool = False,
+) -> int:
+    flags = 0
+    if no_config:
+        flags |= lib.LYD_IMPLICIT_NO_CONFIG
+    if no_state:
+        flags |= lib.LYD_IMPLICIT_NO_STATE
+    if no_defaults:
+        flags |= lib.LYD_IMPLICIT_NO_DEFAULTS
+    if output:
+        flags |= lib.LYD_IMPLICIT_OUTPUT
+    return flags
+
+
 # -------------------------------------------------------------------------------------
 def printer_flags(
     with_siblings: bool = False,
@@ -205,7 +224,7 @@ class DNode:
     Data tree node.
     """
 
-    __slots__ = ("context", "cdata")
+    __slots__ = ("context", "cdata", "free_func")
 
     def __init__(self, context: "libyang.Context", cdata):
         """
@@ -216,6 +235,7 @@ class DNode:
         """
         self.context = context
         self.cdata = cdata  # C type: "struct lyd_node *"
+        self.free_func = None  # type: Callable[DNode]
 
     def meta(self):
         ret = {}
@@ -229,12 +249,57 @@ class DNode:
             item = item.next
         return ret
 
+    def get_meta(self, name):
+        item = self.cdata.meta
+        while item != ffi.NULL:
+            if c2str(item.name) == name:
+                return c2str(
+                    lib.lyd_value_get_canonical(
+                        self.context.cdata, ffi.addressof(item.value)
+                    )
+                )
+            item = item.next
+        return None
+
     def meta_free(self, name):
         item = self.cdata.meta
         while item != ffi.NULL:
             if c2str(item.name) == name:
                 lib.lyd_free_meta_single(item)
                 break
+            item = item.next
+
+    def new_meta(self, name: str, value: str, clear_dflt: bool = False):
+        ret = lib.lyd_new_meta(
+            ffi.NULL,
+            self.cdata,
+            ffi.NULL,
+            str2c(name),
+            str2c(value),
+            clear_dflt,
+            ffi.NULL,
+        )
+        if ret != lib.LY_SUCCESS:
+            raise self.context.error("cannot create meta")
+
+    def add_defaults(
+        self,
+        no_config: bool = False,
+        no_defaults: bool = False,
+        no_state: bool = False,
+        output: bool = False,
+    ):
+        flags = implicit_flags(
+            no_config=no_config,
+            no_defaults=no_defaults,
+            no_state=no_state,
+            output=output,
+        )
+        node_p = ffi.new("struct lyd_node **")
+        node_p[0] = self.cdata
+        ret = lib.lyd_new_implicit_all(node_p, self.context.cdata, flags, ffi.NULL)
+        if ret != lib.LY_SUCCESS:
+            raise self.context.error("cannot get module")
 
     def flags(self):
         ret = {"default": False, "when_true": False, "new": False}
@@ -484,6 +549,22 @@ class DNode:
 
         return DNode.new(self.context, node[0])
 
+    def merge_module(
+        self,
+        source: "DNode",
+        defaults: bool = False,
+        destruct: bool = False,
+        with_flags: bool = False,
+    ) -> None:
+        flags = merge_flags(defaults=defaults, destruct=destruct, with_flags=with_flags)
+        node_p = ffi.new("struct lyd_node **")
+        node_p[0] = self.cdata
+        ret = lib.lyd_merge_module(
+            node_p, source.cdata, ffi.NULL, ffi.NULL, ffi.NULL, flags
+        )
+        if ret != lib.LY_SUCCESS:
+            raise self.context.error("merge failed")
+
     def merge(
         self,
         source: "DNode",
@@ -733,9 +814,9 @@ class DNode:
             elif node.schema.nodetype == SNode.LEAFLIST:
                 if name not in parent_dic:
                     parent_dic[name] = _init_yang_list(node.schema)
-                parent_dic[name].append(DLeaf.cdata_leaf_value(node))
+                parent_dic[name].append(DLeaf.cdata_leaf_value(node, self.context))
             elif node.schema.nodetype == SNode.LEAF:
-                parent_dic[name] = DLeaf.cdata_leaf_value(node)
+                parent_dic[name] = DLeaf.cdata_leaf_value(node, self.context)
 
         dic = {}
         dnode = self
@@ -789,12 +870,18 @@ class DNode:
             rpcreply=rpcreply,
         )
 
+    def free_internal(self, with_siblings: bool = True) -> None:
+        if with_siblings:
+            lib.lyd_free_all(self.cdata)
+        else:
+            lib.lyd_free_tree(self.cdata)
+
     def free(self, with_siblings: bool = True) -> None:
         try:
-            if with_siblings:
-                lib.lyd_free_all(self.cdata)
+            if self.free_func:
+                self.free_func(self)  # pylint: disable=not-callable
             else:
-                lib.lyd_free_tree(self.cdata)
+                self.free_internal(with_siblings)
         finally:
             self.cdata = None
 
@@ -867,10 +954,10 @@ class DList(DContainer):
 @DNode.register(SNode.LEAF)
 class DLeaf(DNode):
     def value(self) -> Any:
-        return DLeaf.cdata_leaf_value(self.cdata)
+        return DLeaf.cdata_leaf_value(self.cdata, self.context)
 
     @staticmethod
-    def cdata_leaf_value(cdata) -> Any:
+    def cdata_leaf_value(cdata, context: "libyang.Context" = None) -> Any:
 
         val = lib.lyd_get_value(cdata)
         if val == ffi.NULL:
@@ -881,8 +968,9 @@ class DLeaf(DNode):
         val_type = ffi.new("const struct lysc_type **", ffi.NULL)
 
         # get real value type
+        ctx = context.cdata if context else ffi.NULL
         ret = lib.lyd_value_validate(
-            ffi.NULL,
+            ctx,
             term_node.schema,
             str2c(val),
             len(val),
@@ -893,10 +981,18 @@ class DLeaf(DNode):
 
         if ret in (lib.LY_SUCCESS, lib.LY_EINCOMPLETE):
             val_type = val_type[0].basetype
-            if val_type == Type.BOOL:
-                return val == "true"
+            if val_type in Type.STR_TYPES:
+                return val
             if val_type in Type.NUM_TYPES:
                 return int(val)
+            if val_type == Type.BOOL:
+                return val == "true"
+            if val_type == Type.DEC64:
+                return float(val)
+            if val_type == Type.LEAFREF:
+                return DLeaf.cdata_leaf_value(cdata.value.leafref, context)
+            if val_type == Type.EMPTY:
+                return None
             return val
 
         raise TypeError("value type validation error")
@@ -1033,7 +1129,7 @@ def dict_to_dnode(
             str2c(name),
             in_rpc_output,
             n,
-            *[str2c(i) for i in key_values],
+            *[str2c(str(i)) for i in key_values],
         )
         if ret != lib.LY_SUCCESS:
             if _parent:
