@@ -793,6 +793,9 @@ class IfFeature(IfFeatureExprTree):
     def feature(self) -> Feature:
         return Feature(self.context, self.cdata)
 
+    def state(self) -> bool:
+        return self.feature().state()
+
     def dump(self, indent: int = 0) -> str:
         feat = self.feature()
         return "%s%s [%s]\n" % (" " * indent, feat.name(), feat.description())
@@ -810,6 +813,9 @@ class IfNotFeature(IfFeatureExprTree):
         self.context = context
         self.child = child
 
+    def state(self) -> bool:
+        return not self.child.state()
+
     def dump(self, indent: int = 0) -> str:
         return " " * indent + "NOT\n" + self.child.dump(indent + 1)
 
@@ -829,6 +835,9 @@ class IfAndFeatures(IfFeatureExprTree):
         self.a = a
         self.b = b
 
+    def state(self) -> bool:
+        return self.a.state() and self.b.state()
+
     def dump(self, indent: int = 0) -> str:
         s = " " * indent + "AND\n"
         s += self.a.dump(indent + 1)
@@ -851,6 +860,9 @@ class IfOrFeatures(IfFeatureExprTree):
         self.a = a
         self.b = b
 
+    def state(self) -> bool:
+        return self.a.state() or self.b.state()
+
     def dump(self, indent: int = 0) -> str:
         s = " " * indent + "OR\n"
         s += self.a.dump(indent + 1)
@@ -1035,17 +1047,17 @@ class SLeaf(SNode):
         self.cdata_leaf = ffi.cast("struct lysc_node_leaf *", cdata)
         self.cdata_leaf_parsed = ffi.cast("struct lysp_node_leaf *", self.cdata_parsed)
 
-    def default(self) -> Optional[str]:
+    def default(self) -> Union[None, bool, int, str]:
         if not self.cdata_leaf.dflt:
             return None
         val = lib.lyd_value_get_canonical(self.context.cdata, self.cdata_leaf.dflt)
         if not val:
             return None
         val = c2str(val)
-        val_type = self.cdata_leaf.dflt.realtype
-        if val_type == Type.BOOL:
+        val_type = Type(self.context, self.cdata_leaf.dflt.realtype, None)
+        if val_type.base() == Type.BOOL:
             return val == "true"
-        if val_type in Type.NUM_TYPES:
+        if val_type.base() in Type.NUM_TYPES:
             return int(val)
         return val
 
@@ -1085,7 +1097,7 @@ class SLeafList(SNode):
         )
 
     def ordered(self) -> bool:
-        return bool(self.cdata.flags & lib.LYS_ORDBY_USER)
+        return bool(self.cdata_parsed.flags & lib.LYS_ORDBY_USER)
 
     def units(self) -> Optional[str]:
         return c2str(self.cdata_leaflist.units)
@@ -1169,7 +1181,7 @@ class SList(SNode):
         self.cdata_list_parsed = ffi.cast("struct lysp_node_list *", self.cdata_parsed)
 
     def ordered(self) -> bool:
-        return bool(self.cdata.flags & lib.LYS_ORDBY_USER)
+        return bool(self.cdata_parsed.flags & lib.LYS_ORDBY_USER)
 
     def __iter__(self) -> Iterator[SNode]:
         return self.children()
